enum Operation { compress, expand }

abstract class Archiver {
  /// returns the archival extension that this [Archiver]
  /// supports
  String get extension;

  /// True if the platform command required for this
  /// [Archiver] is installed for the passed operation.
  bool isInstalled(Operation operation);

  /// returns true if this archive supports compression
  bool get canCompress;

  /// returns true if this archive supports expansion
  bool get canExpand;

  /// returns the name of the platform command used to do the archiving
  /// for the given [operation] type (compress or expand)
  String command(Operation operation);

  /// The archive file [pathToArchive] will be created by compressing the
  /// contents of the [workingDirectory].
  void compress(
      {required String pathToArchive,
      required String workingDirectory,
      required bool recursive});

  /// The archive file located at [pathToArchive] will be expanded
  /// into [workingDirectory]
  void expand(
      {required String pathToArchive, required String workingDirectory});

  /// The archive file located at [pathToArchive] will be expanded
  /// into a temporary directory and the file within the archive located at
  /// [pathToSingle] will be extracted into [workingDirectory].
  /// If [includePath] is false (default) then the path will be stripped
  /// from the extracted single file and it will be stored directly under
  /// [workingDirectory] without a path prefix.
  void expandSingle(
      {required String pathToArchive,
      required String workingDirectory,
      required String pathToSingle,
      bool includePath = false});
}
