import 'archiver.dart';
import 'rar.dart';
import 'tar.dart';
import 'tar_gz.dart';
import 'xz.dart';
import 'zip.dart';

class Archivers {
  factory Archivers() => _self;

  Archivers._internal();

  static late final _self = Archivers._internal();

  final archivers = <Archiver>[Rar(), TarGz(), Tar(), Xz(), Zip()];

  String get usageSupported {
    final sb = StringBuffer();

    for (final archive in archivers) {
      sb.writeln(archive.extension);
    }
    return sb.toString();
  }

  /// Finds the archiver that supports the archive file
  /// located at [pathToArchive].
  /// The archive's extension is used to determine the
  /// appropriate archiver.
  Archiver? determineArchiver(String pathToArchive) {
    /// try to match the longest extension first so that
    /// tar.gz takes precendence of .gz
    for (final archiver in archivers
      ..sort((lhs, rhs) => lhs.extension.length - rhs.extension.length)) {
      if (pathToArchive.endsWith(archiver.extension)) {
        return archiver;
      }
    }
    return null;
  }
}
