import 'package:args/command_runner.dart';
import 'package:dcli/dcli.dart';
import '../archivers/archiver.dart';
import '../archivers/archivers.dart';

class Compress extends Command<int> {
  Compress() {
    argParser
      ..addOption('from',
          abbr: 'f',
          defaultsTo: '.',
          help: 'The file or directory to compress.')
      ..addOption('name',
          abbr: 'n',
          mandatory: true,
          help: 'The name of the archive to create including its extension. '
              'The extension is used to determine the archive type.')
      ..addFlag('recursive',
          abbr: 'r',
          defaultsTo: true,
          help: "Indicates whether the subdirectories in 'from' "
              'should be added to the archive .');
  }
  @override
  String get description => 'Compresses a directory into an archive file';

  @override
  String get name => 'compress';

  @override
  int run() {
    final name = argResults!['name'] as String;
    final recursive = argResults!['recursive'] as bool;
    final from = argResults!['from'] as String;

    if (!exists(from)) {
      printerr(red("The 'from' directory $from doesn't exist."));
      return 1;
    }

    if (exists(name)) {
      if (!confirm('Overwrite the existing $name?')) {
        return 1;
      }
    }

    final archiver = Archivers().determineArchiver(name);
    if (archiver == null) {
      printerr(red("The archive type '${extension(name)}' isn't supported."));
      return 1;
    }

    if (!archiver.canCompress) {
      printerr(red('The archiver for ${archiver.extension} '
          'does not support compression'));
    }

    if (!archiver.isInstalled(Operation.compress)) {
      printerr(red("The archiver for '${archiver.extension}' "
          "is supported but isn't currently installed. "
          'Install ${archiver.command(Operation.compress)} and try again'));
      return 1;
    }

    archiver.compress(
        pathToArchive: name, workingDirectory: from, recursive: recursive);

    return 0;
  }
}
