@Timeout(Duration(minutes: 10))
import 'package:darchive/src/darchive.dart';
import 'package:dcli/dcli.dart' hide equals;
import 'package:test/test.dart';

void main() {
  test('darchive tar expand', () {
    withTempDir((archiveDir) {
      final pathToArchive = createTar(archiveDir, 'test.tar');
      withTempDir((tmp) {
        final expandTo = join(tmp, 'expanded');
        expect(
            entryPoint(['expand', '--name=$pathToArchive', '--to=$expandTo']),
            equals(0));
        expect(exists(one(expandTo)), isTrue);
        expect(exists(two(expandTo)), isTrue);
        expect(exists(three(expandTo)), isTrue);
      });
    });
  });

  test('darchive tar compress', () {
    withTempDir((sourceDir) {
      createTree(sourceDir);
      withTempDir((compressTo) {
        final pathToArchive = join(compressTo, 'test.tar');
        expect(
            entryPoint(
                ['compress', '--name=$pathToArchive', '--from=$sourceDir']),
            equals(0));

        withTempDir((tmp) {
          final expandTo = join(tmp, 'expanded');
          expect(
              entryPoint(['expand', '--name=$pathToArchive', '--to=$expandTo']),
              equals(0));
          expect(exists(one(expandTo)), isTrue);
          expect(exists(two(expandTo)), isTrue);
          expect(exists(three(expandTo)), isTrue);
        });
      });
    });
  });

  test('darchive tar.gz compress/expand', () {
    withTempDir((sourceDir) {
      createTree(sourceDir);
      withTempDir((compressTo) {
        final pathToArchive = join(compressTo, 'test.tar.gz');
        expect(
            entryPoint(
                ['compress', '--name=$pathToArchive', '--from=$sourceDir']),
            equals(0));

        withTempDir((tmp) {
          final expandTo = join(tmp, 'expanded');
          expect(
              entryPoint(['expand', '--name=$pathToArchive', '--to=$expandTo']),
              equals(0));
          expect(exists(one(expandTo)), isTrue);
          expect(exists(two(expandTo)), isTrue);
          expect(exists(three(expandTo)), isTrue);
        });
      });
    });
  });

  test('darchive xz compress/expand', () {
    withTempDir((sourceDir) {
      createTree(sourceDir);
      withTempDir((compressTo) {
        final pathToArchive = join(compressTo, 'test.xz');
        expect(
            entryPoint(
                ['compress', '--name=$pathToArchive', '--from=$sourceDir']),
            equals(0));

        withTempDir((tmp) {
          final expandTo = join(tmp, 'expanded');
          expect(
              entryPoint(['expand', '--name=$pathToArchive', '--to=$expandTo']),
              equals(0));
          expect(exists(one(expandTo)), isTrue);
          expect(exists(two(expandTo)), isTrue);
          expect(exists(three(expandTo)), isTrue);
        });
      });
    });
  });

  test('darchive zip compress/expand', () {
    withTempDir((sourceDir) {
      createTree(sourceDir);
      withTempDir((compressTo) {
        final pathToArchive = join(compressTo, 'test.zip');
        expect(
            entryPoint(
                ['compress', '--name=$pathToArchive', '--from=$sourceDir']),
            equals(0));

        withTempDir((tmp) {
          final expandTo = join(tmp, 'expanded');
          expect(
              entryPoint(['expand', '--name=$pathToArchive', '--to=$expandTo']),
              equals(0));
          expect(exists(one(expandTo)), isTrue);
          expect(exists(two(expandTo)), isTrue);
          expect(exists(three(expandTo)), isTrue);
        });
      });
    });
  });
}

String one(String parent) => join(parent, 'one.txt');
String two(String parent) => join(parent, 'two.txt');
String three(String parent) => join(parent, 'subdir' 'three.txt');

String createTar(String archiveDir, String archiveName) =>
    withTempDir((pathToContent) {
      createTree(pathToContent);
      final pathToArchive = join(archiveDir, archiveName);
      'tar -cf $pathToArchive .'.start(workingDirectory: pathToContent);
      return pathToArchive;
    });

void createTree(String pathToContent) {
  touch(one(pathToContent), create: true);
  touch(two(pathToContent), create: true);
  touch(three(pathToContent), create: true);
}
